#!/usr/bin/perl

# Author: Bodo Eggert <7eggert@gmx.de>
# Status: pre-alpha
# License: GPL

#
# To use the script, make a subdirectory and save (or symlink)
# it there. This will be the script data directory.
# Adjust the path to your napshare temporary directory,
# any file not blacklisted and existing in that directory will be
# downloaded.
#
# While used as a filter, the filenames are normalized, matched
# against some filters and, depending on the result, written to
# 1.txt (new), 2.txt or 3.txt (different blocklists) as long as
# they are neither in f_getlist or f_blocklist.
#

($cwd=$0)=~s/\/[^\/]*$//;
chdir $cwd;

#begin program variables
 $GETLIST="/home/7eggert/mp3/Getlist.napshare";
 $GETLIST_REMOVE="/home/7eggert/mp3/Getlist.napshare.removeme";
 $BLOCKLIST="/home/7eggert/mp3/Blacklist.napshare";
 $MP3_BLOCKLIST_ARTIST="/home/7eggert/mp3/Blacklist.artist";
 $MP3_ALBUM="/home/7eggert/mp3/Album";
 $MP3_COMPOSE="/home/7eggert/mp3/Compose.mp3";
 $MP3_A_SPELL="/home/7eggert/mp3/Spell.artist";
 $NAPDIR="/home/7eggert/scratch/napshare/tmp/";	#don't omit the slash!
#end program variables

#begin system variables
 $FGREP="/usr/bin/fgrep";
#end system variables

$block_f="^(!|how to |(documents )?read ?me|really works\!|(preview - )?t - [0-9]{7}|n[0-9]{2} - [0-9]{7})|\\[VBR100%?\\]|\.(asf|bpl|doc|framepos|html?|kopi(a av|e van)|lnk|m3u|mp2|ots|php|played|ram|rm[jx]|sfk|te?mp|url|vbs|wav|wm[av]|pk)\$";
%extfix=('mp3+','mp3','divx','avi','mpg','mpeg');
$genres="[\\&+] *|[3-9]0(s|er)( ?- ?[3-9]0(s|er))?( music)?|alternative|burned mp3|children video|classics?|comedy|copy (\([0-9]+\) )?of|cpia de|dance|easy listening|gospel|kopie von|misc|mp3s?|my (files|music|shared folder)|napster|new (pop|wave)|non (- checked( albums( (in)?complete)?)?|album tracks)|ny musikk|ok|oldies?( but goldies)?|please share|pop(-80s)?|punk( covers?)?|rap|rare bootleg|reggae|rock( [\\&n] roll( christmas))?|sixties|slow songs|so?u?ndtra?c?ks?( movies)?|techno|to burn|un(known|titled)|track [0-9]{1,2}|uploads|vario?us( artists?)?|xmas";

sub loadalbums {
	%album=();
	my $fname=shift;
	open(F,'<',$fname) or die "Can't open '$fname': $!\n";
	while(<F>){
		s/\n//;
		$_=lc;
		$n=split(' - ');
		my $band = $_[0];
		for(my $i=1;$i<$n;$i++) {
			$album{($band,$_[$i])}=true;
}	}	}

sub loadCompose {
	%compose=();
	my $fname=shift;
	open(F,'<',$fname) or die "Can't open '$fname': $!\n";
	while(<F>){
		s/\n//;
		$_=lc;
		$n=split(' - ');
		my $c = $_[0];
		for(my $i=2;$i<$n;$i+=2) {
			$compose{($_[$i-1],$_[$i])}=$c;
}	}	}

sub loadspell {
	%spell=();
	my $fname=shift;
	open(F,'<',$fname) or die "Can't open '$fname': $!\n";
	while(<F>){
		s/\n//;
		$_=lc;
		$n=split(' - ');
		for(my $i=1;$i<$n;$i++){$spell{$_[$i]}=$_[0];}
	}
}

$mode=' ';
$instring = $ARGV[0];
if(($instring=~s/^-(.)$/$1/)==1){
	$mode=$instring;
	$instring = $ARGV[1];
}

($ofilename, $size, $speed, $stat, $tag) = split(/\//,$instring,5);

$restart=2;
$status=1;

$file_in_napdir=0;
if($mode=~/[^A-Z]/ && -e $NAPDIR.$ofilename) {
	$file_in_napdir=1;
}

	if($ofilename =~ /$block_f/i) {
		$status=3;
		$filename=$ofilename;
	}
check: while(1) {
	if($status=1){
		for($filename=lc($ofilename)){
			 #replace some junk characters and %20 with spaces/dashes
			s/[\^=]|-=-|___/-/g;
			if(!/-/){s/[_\#~]/-/g;}
			s/[_\#~]|%20/ /g;
			if(/[^-+]*\+[^-+]*/){s/\+/-/};
			 #remove other junk characters and tailing spaces, leading ones are done later
			s/\'|\`|| +$|\(\!\)|\\//g;
			 #strip more tha 4 occurences (sp?) of characters
			s/(.)\1{3,}/$1$1$1/g;
			 #replace '{}[]' with '()()'
			tr/\{\}\[\]/\(\)\(\)/;
			 #more junk
			s/ *sampleaccurate.com\.[a-z0-9]+$//i;
			s/\(found via www\.fileDonkey\.com\)//i;
			s/\.(sharereactor|\(www\.the-realworld\.de\))\././i;
			 #extension mangeling
			while ( ($key, $value) = each %extfix)
			{
				s/\.$key$/\.$value$1/i;
				s/\.$key(\.[a-z0-9]*)?$/\.$value$1/i;
			}
			 #if there are no spaces, change dots into spaces
			if(!/ /) {
				while(s/\.(.*)\./ $1./g){};
			}
			 #enforce spaces around dashes
			s/-/ - /g;
			 # add spaces outside '()'
			s/\(/ \(/;s/\)/\) /;
			 #remove double extensions
			s/(\.[a-z0-9]*)\1$/$1/i;
			 #remove tailing "(1)"to"(99)"-sequences like in "filename (2)(1).foo"
			s/( *\([0-9]{1,2}\)| *[-+])+ *(\.[0-9a-z]*)$/$2/i;		 
			
			 #remove double spaces and spaces after (
			s/([\( ]) +/$1/g;
			 #remove spaces and dashes before ) and ,.
			s/[- ]+([\,\.\)])/$1/g;
			 #remove spaces before extension
			s/ *\.+([0-9A-Z]*)$/\.$1/i;

			 #remove [-=,.] before " - "
			s/[-=,\.]+ - / - /g;
			 #remove "- [0-9]* -"
			s/- [0-9]* - /- /;
			 #remove multiple "- "
			s/(- )+/- /g;
			 #remove leading spaces and dashes
			do {	s/^[-,.= ]*//;
			 #remove genre text
			} until((s/^(($genres)\b|\(($genres)\)|\[($genres)\]|\{($genres)\})//io)==0);
			s/^[-= ]*//;
			 #remove () around first word, replacing them with ' - '
			 #we're less strict than with genre, since no text is erased
			while(s/^\(([A-Z0-9., \&]*)\)/$1 - /i > 0
			|| s/ - \(([A-Z0-9., \&]*)\) - / - $1 - /i > 0){
				s/([\( ]) +/$1/g;	  #double spaces and dashes after '('
				s/[- ]+\)/\)/g;   #dashes and spaces before ')'
				s/[-=,\.]* - / - /g;	  #some chars before " - "
			}
			 # replace " - 1of2" and "(1 of 2)" with " - 1 of 2"
			s/- ([0-9]*) *of *([0-9]*)\b/- $1 of $2/g;
			s/\(([0-9]*) *of *([0-9]*)\)/ - $1 of $2/g;

			 #dashes before '.'
			s/[- ]+\./\./g;
			 #double dashes
			s/(- )+/- /g;
			 #dashes at start of string
			s/^ -? ?//g;

			if($restart>0) {
				if((!/\(/ && (s/\)/-/>0))
				|| (!/\)/ && (s/\(/-/>0))) {
					$restart--;
					$ofilename=$filename;
					next check;
				}
			}
		}
	}

	$ofilename=$filename;
	if($mode=~/[^A-Z]/) {
		if(!system $FGREP, '-qsxi', '--', $filename, $GETLIST) {
			$status=0;
			last check;
		} elsif(!system $FGREP, '-qsxi', '--', $filename, $BLOCKLIST){
			$status=18;
			last check;
		}
	}
##################################
	if ($filename =~ /\.mp3$|\.ogg/i) {

		mp3_check: while(1){
			$blocklist_f = "remix\\b|\\b(complete|entire) (album|cd)\\b|\\(cd\\)";

			 #remove specification of the mix
			$filename =~ s/ ?\([A-Z]* mix\)/(mix)/i;
			$filename =~ s/ ?\(([A-Z ]* )?cover\) ?//i;
			$filename =~ s/ ?\(Rare\) ?//i;
			$filename =~ s/\([A-Z ]* parody\)/(parody)/i;
			$filename =~ s/\(parody of [A-Z ]*\)/(parody)/i;
			$filename =~ s/ ?\(live\)$/ - live/;
			$filename =~ s/ +/ /g;
			($ext=$filename) =~ s/.*(\.mp3|\.ogg)$/$1/i;
			($f=$filename) =~ s/(\.mp3|\.ogg)*$//i;	
			if ($f=~ /$blocklist_f/i) {
				$status=2;
				last mp3_check;
			}
			$f =~ s/\!s$//;
			loadspell($MP3_A_SPELL);
			loadalbums($MP3_ALBUM);
			loadCompose($MP3_COMPOSE);

			@test = split(/ - /, $f, -1);
			for($i=0;$i<@test;$i++){
				if($test[$i] eq ""){splice(@test,$i,1);}
			}
			if(@test==1) {
				@test = split(/ *[\(\)] */, $f, -1);
				for($i=0;$i<@test;$i++){
					if($test[$i] eq ""){splice(@test,$i,1);$i--;}
			}	}
			if(@test==1) {
				if($test[0]=~/^[^,]*,+[^,]*$/) {
					@test = split(/,+/, $test[0], -1);
				} elsif($test[0]=~/^[^\.]*\.+[^\.]*$/) {
					@test = split(/\.+/, $test[0], -1);
				} elsif($test[0]=~/^[^=]*=+[^=]*$/) {
					@test = split(/=+/, $test[0], -1);
				} elsif($test[0]=~/^[^ ]* +[^ ]*$/) {
					@test = split(/ +/, $test[0], -1);
				}
				for($i=0;$i<@test;$i++){
					if($test[$i] eq ""){splice(@test,$i,1);$i--;}
			}	}

			if(@test > 2) {
				for($i=0;$i<@test;$i++){
					if($test[$i] =~ m/^[0-9]*$/){splice(@test,$i,1);$i--;}
			}	}
			for ($i=1;$i<@test;$i++) {
				if($test[$i] eq $test[$i-1] && $test[$i] =~ / /){splice(@test,$i,1);}
			}
			$test[0] =~ s/^([^,]*?),? ?the$/the $1/;
			$test[0] =~ s/^([^,]*?),? ?die$/die $1/;
			$test[0] =~ s/ +/ /g;

			$a=$spell{lc($test[0])};
			if($a){$test[0]=$a;}

			if(@test > 2) {
				my $c=$compose{($test[0],$test[1])};
				if($c){
					splice(@test,0,2,($c));
				} elsif(@test > 3) {
					$c=$compose{(@test[1],@test[2])};
					if($c && $album{($c,$test[0])}){splice(@test,0,3,($c));}
			}	}
			if(@test > 2) {
				if($album{($test[0],$test[1])}) {splice(@test,1,1);$i--;}
				elsif($album{($test[1],$test[0])}) {splice(@test,0,1);$i--;}
				else {
					$a=$spell{lc($test[1])};
					if($a && $album{($a,$test[0])}) {
						splice(@test,0,1);
						$test[0]=$a;
			}	}	}
			
			if(@test == 2) {
				if($test[1] =~ s/ \((live)\)//i){$test[2]=$1};
			}

			last mp3_check;
		}	#:mp3_check

		$filename = join(' - ',@test).'.mp3';
		$ofilename = join(' - ',@test).$ext;

		if($mode=~/[^A-Z]/) {
			if(!system $FGREP, '-qsxi', '--', $test[0], $MP3_BLOCKLIST_ARTIST) {
				$status=19;
			} elsif(!system $FGREP, '-qsxi', '--', $filename, $GETLIST) {
				$status=0;
			} elsif(!system $FGREP, '-qsxi', '--', $filename, $BLOCKLIST){
				$status=18;
		}	}

		last check;
##################################
	}
	last check;
}	#:check
$filename =~ s/(^|[^a-zA-Z0-9\200-\377])([a-z])/$1\u$2/g;
$filename =~ s/\.([a-zA-Z0-9.]*)$/\.\L$1\E/;

$ofilename =~ s/(^|[^a-zA-Z0-9\200-\377])([a-z])/$1\u$2/g;
$ofilename =~ s/\.([a-zA-Z0-9.]*)$/\.\L$1\E/;

if(($file_in_napdir == 1)&&($status==1)) {$status=0;}

$mode=lc($mode);
if($status == 0 && $mode eq ' ') { exit 0;}

if(($status<16)or($mode ne " ")){
	 #default
	if($mode eq ' '){$LogFile = $status.'.txt';}
	 #test or print
	elsif($mode =~ /[pt]/){$LogFile='/dev/stdout';}
	 #blacklist
	elsif ($mode eq b){
		$LogFile=$BLOCKLIST;
		if($status==0){
			((-e $GETLIST_REMOVE)
				? open(LOG,">>$GETLIST_REMOVE")
				: open(LOG,">$GETLIST_REMOVE"))
				|| die("Can't open $GETLIST_REMOVE: $!\n");
			print LOG $filename."\n";
			close(LOG);
		} elsif($status>1){
			exit $status!=0;
		}
	}
	 #getlist
	elsif ($mode eq g){
		$LogFile=$GETLIST;
		if($status!=1){exit $status!=0;}
	}
	else {print "Usage: command -[tpbg] filestring\n\tt:test\n\tp:print\n\tb:blacklist\n\tg:getlist\n"; exit 126;}
	((-e $LogFile)? open(LOG,">>$LogFile"):open(LOG,">$LogFile"))
	  || die("Can't open $LogFile: $!\n");

	if($mode eq 'p') {print LOG (($mode eq 't')?$status .'/':'').$ofilename."\n";}
	else {print LOG (($mode eq 't')?$status .'/':'').$filename."\n";}
} else { print $status .'/'.$filename."\n"; }

exit $status!=0; 
